@@ -2,7 +2,7 @@
 #include <stdlib.h>
 #ifdef WIN32
 #include <windows.h>
-#include <Lmcons.h>
+#include <lmcons.h>
 #include <strsafe.h>
 #include <log.h>
 #include <qubes-io.h>
@@ -48,7 +48,7 @@ struct qdb_handle {
     struct path_list *watch_list;
 };
 
-void free_path_list(struct path_list *plist) {
+static void free_path_list(struct path_list *plist) {
     struct path_list *tmp;
 
     while (plist) {
@@ -74,7 +74,7 @@ static int connect_to_daemon(struct qdb_handle *qh) {
         WCHAR user_name[user_name_len];
 
         if (!GetUserName(user_name, &user_name_len)) {
-            perror("GetUserName");
+            win_perror("GetUserName");
             return 0;
         }
         StringCbPrintf(pipe_name, sizeof(pipe_name), QDB_DAEMON_LOCAL_PATH, user_name);
@@ -90,7 +90,7 @@ static int connect_to_daemon(struct qdb_handle *qh) {
     status = QpsConnect(pipe_name, &qh->read_pipe, &qh->write_pipe);
     if (status != ERROR_SUCCESS)
     {
-        perror2(status, "connect to server");
+        // win_perror2(status, "connect to server");
         return 0;
     }
 
@@ -162,7 +162,7 @@ static int send_command_to_daemon(qdb_handle_t h, struct qdb_hdr *hdr, void *dat
             else {
                 /* try again */
                 if (!QioWriteBuffer(h->write_pipe, hdr, sizeof(*hdr))) {
-                    perror("write to daemon");
+                    win_perror("write to daemon");
                     return 0;
                 }
                 else
@@ -170,14 +170,14 @@ static int send_command_to_daemon(qdb_handle_t h, struct qdb_hdr *hdr, void *dat
             }
         } else {
             /* other write error */
-            perror("write to daemon");
+            win_perror("write to daemon");
             return 0;
         }
     }
     if (data && !QioWriteBuffer(h->write_pipe, data, hdr->data_len)) {
         /* no recovery after header send, daemon most likely closed connection
          * in reaction to our data */
-        perror("write to daemon");
+        win_perror("write to daemon");
         return 0;
     }
     return 1;
@@ -367,9 +367,9 @@ char *qdb_read(qdb_handle_t h, char *path, unsigned int *value_len) {
     hdr.data_len = 0;
     if (!send_command_to_daemon(h, &hdr, NULL))
         /* some fatal error perhaps? */
-        return NULL;
+        goto fail;
     if (!get_response(h, &hdr))
-        return NULL;
+        goto fail;
     /* TODO: make this distinguishable from other errors */
     if (hdr.type == QDB_RESP_ERROR_NOENT) {
         errno = ENOENT;
@@ -378,13 +378,13 @@ char *qdb_read(qdb_handle_t h, char *path, unsigned int *value_len) {
     if (hdr.type == QDB_RESP_ERROR) {
         /* TODO? */
         assert(hdr.data_len == 0);
-        return NULL;
+        goto fail;
     }
     assert(hdr.type == QDB_RESP_READ);
     /* +1 for terminating \0 */
     value = malloc(hdr.data_len+1);
     if (!value)
-        return NULL;
+        goto fail;
     got_data = 0;
     while (got_data < hdr.data_len) {
 #ifdef WIN32
@@ -395,7 +395,7 @@ char *qdb_read(qdb_handle_t h, char *path, unsigned int *value_len) {
         if (ret <= 0) {
 #endif
             free(value);
-            return NULL;
+            goto fail;
         }
         got_data += ret;
     }
@@ -405,6 +405,11 @@ char *qdb_read(qdb_handle_t h, char *path, unsigned int *value_len) {
         *value_len = got_data;
 
     return value;
+fail:
+    /* ENOENT is returned for "no such path in QubesDB" */
+    if (errno == ENOENT || errno == 0)
+        errno = EPROTO;
+    return NULL;
 }
 
 char **qdb_list(qdb_handle_t h, char *path, unsigned int *list_len) {
