@@ -5,8 +5,10 @@
 #include <unistd.h>
 #else
 #define strdup _strdup
+#include <windows.h>
 #include <strsafe.h>
 #endif
+#include "buffer.h"
 
 #include <qubesdb.h>
 #include "qubesdb_internal.h"
@@ -51,12 +53,12 @@ void qubesdb_destroy(struct qubesdb *db) {
 
         tmp_entry->prev->next = tmp_entry->next;
         tmp_entry->next->prev = tmp_entry->prev;
-        memset(tmp_entry->value, 0, tmp_entry->value_len);
+        buffer_secure_zero(tmp_entry->value, tmp_entry->value_len);
         free(tmp_entry->value);
         free(tmp_entry);
     }
 
-    memset(db->entries->value, 0, db->entries->value_len);
+    buffer_secure_zero(db->entries->value, db->entries->value_len);
     free(db->entries->value);
     free(db->entries);
     // TODO: free watches
@@ -135,7 +137,7 @@ int qubesdb_write(struct qubesdb *db, char *path, char *data, int data_len) {
         return 0;
     }
     if (db_entry->value) {
-        memset(db_entry->value, 0, db_entry->value_len);
+        buffer_secure_zero(db_entry->value, db_entry->value_len);
         free(db_entry->value);
     }
     if (data_len) {
@@ -144,6 +146,10 @@ int qubesdb_write(struct qubesdb *db, char *path, char *data, int data_len) {
     } else {
         db_entry->value = strdup("");
     }
+    if (!db_entry->value) {
+        fputs("malloc(3) failed\n", stderr);
+        _exit(1);
+    }
     db_entry->value_len = data_len;
     return 1;
 }
@@ -173,7 +179,7 @@ int qubesdb_remove(struct qubesdb *db, char *path) {
 
         tmp_entry->prev->next = tmp_entry->next;
         tmp_entry->next->prev = tmp_entry->prev;
-        memset(tmp_entry->value, 0, tmp_entry->value_len);
+        buffer_secure_zero(tmp_entry->value, tmp_entry->value_len);
         free(tmp_entry->value);
         free(tmp_entry);
         anything_removed = 1;
