@@ -4,13 +4,21 @@
 
 #include <libvchan.h>
 
-#ifndef WIN32
 #include "buffer.h"
-#else
+#ifdef WIN32
 #include <log.h>
 #include <pipe-server.h>
+#include <qubesdb.h>
 #endif
 
+#include <assert.h>
+
+#define QUBES_FD_SET(a, b) do { \
+    int x = (a); \
+    assert((x >= 0 && x < (FD_SETSIZE))); \
+    FD_SET(x, (b)); \
+} while (0)
+
 #ifndef WIN32
 typedef int client_socket_t;
 #define INVALID_CLIENT_SOCKET -1
@@ -73,12 +81,11 @@ struct db_daemon_data {
     int remote_connected;       /* if remote daemon connected and ready for
                                  * processing requests (i.e. have
                                  * synchronised database */
-    struct buffer *vchan_buffer;/* vchan write buffer */
-    struct qdb_hdr vchan_pending_hdr; /* retrieved header but data not yet handled */
 #ifdef WIN32
     PIPE_SERVER pipe_server;
     SECURITY_ATTRIBUTES sa;
     HANDLE service_stop_event;
+    SRWLOCK lock;
 #else
     int socket_fd;              /* local server socket */
     struct client *client_list; /* local clients */
@@ -89,6 +96,8 @@ struct db_daemon_data {
     struct qubesdb *db;         /* database */
     int multiread_requested;    /* have requested multiread, if not - drop such
                                    responses */
+    struct buffer *vchan_buffer;/* vchan write buffer */
+    struct qdb_hdr vchan_pending_hdr; /* retrieved header but data not yet handled */
 };
 
 struct qubesdb *qubesdb_init(send_watch_notify_t);
